﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp1
{
    public partial class Editor : Form
    {
        List<Bitmap> rooms = new List<Bitmap>();
        int room = 0;
        int rOffset = 0;
        List<Bitmap> textures = new List<Bitmap>();
        int texture = 0;
        int tOffset = 0;
        List<Color> colors = new List<Color>();
        int cOffset = 0;
        int drawing = 0;
        Point p1 = new Point(0, 0), p2 = new Point(0, 0);
        Bitmap overlay = new Bitmap(50, 50);

        public Editor()
        {
            InitializeComponent();
            typeof(Panel).InvokeMember("DoubleBuffered", BindingFlags.SetProperty | BindingFlags.Instance | BindingFlags.NonPublic, null, roomView, new object[] { true });
        }

        private void Editor_Load(object sender, EventArgs e)
        {
            NewRoom();
            NewTexture();
        }

        #region Rooms
        #region Room management
        //Ovládání palety místností
        private void roomAdd_Click(object sender, EventArgs e)
        {
            NewRoom();
            if (rooms.Count > 5) rOffset = rooms.Count - 5;
            roomPalette.Refresh();
        }

        private void roomRemove_Click(object sender, EventArgs e)
        {
            rooms.RemoveAt(room);
            if (rooms.Count <= 0) NewRoom();
            else if (room >= rooms.Count) room = rooms.Count - 1;
            if (rOffset > 0 && rOffset + 5 > rooms.Count) rOffset--;
            roomView.Refresh();
            roomPalette.Refresh();
        }

        private void NewRoom()
        {
            Bitmap b = new Bitmap(50, 50);
            using (Graphics g = Graphics.FromImage(b)) g.Clear(Color.Magenta);
            rooms.Add(b);
        }

        private void roomPalette_MouseClick(object sender, MouseEventArgs e)
        {
            int m = roomPalette.PointToClient(MousePosition).X / 50 + rOffset;
            if (m < rooms.Count)
            {
                room = m;
                roomView.Refresh();
            }
        }        
        
        private void roomPalette_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            for (int i = 0; i < (rooms.Count < 5 ? rooms.Count : 5); i++)
            {
                g.DrawImage(rooms[i+rOffset], new Point(50 * i, 0));
            }
        }

        private void roomLeft_Click(object sender, EventArgs e)
        {
            Button b = sender as Button;
            if (b.Text == "←" && rOffset > 0) rOffset--;
            else if (b.Text == "→" && rOffset + 5 < rooms.Count) rOffset++;
            roomPalette.Refresh();
        }
        #endregion
        #region Room painting
        //Kreslení místností
        private void roomView_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;
            g.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.Half;
            g.DrawImage(rooms[room], new Rectangle(0, 0, roomView.Width, roomView.Height));
            if (drawing != 0)
            {
                using (Graphics f = Graphics.FromImage(overlay))
                {
                    f.Clear(Color.Transparent);
                    f.DrawLine(new Pen(drawing == 1 ? colorView.BackColor : Color.Magenta), p1, p2);
                }
                g.DrawImage(overlay, new Rectangle(0, 0, roomView.Width, roomView.Height));
            }
        }

        private void roomView_MouseDown(object sender, MouseEventArgs e)
        {
            p1 = new Point(roomView.PointToClient(MousePosition).X / 5, roomView.PointToClient(MousePosition).Y / 5);
            p2 = new Point(roomView.PointToClient(MousePosition).X / 5, roomView.PointToClient(MousePosition).Y / 5);
            if(e.Button == MouseButtons.Left)
            {
                rooms[room].SetPixel(p1.X, p1.Y, colorView.BackColor);
                drawing = 1;
            }
            else if (e.Button == MouseButtons.Right)
            {
                rooms[room].SetPixel(p1.X, p1.Y, Color.Magenta);
                drawing = 2;
            }
        }

        private void roomView_MouseMove(object sender, MouseEventArgs e)
        {
            if (drawing != 0)
            {
                p2 = new Point(roomView.PointToClient(MousePosition).X / 5, roomView.PointToClient(MousePosition).Y / 5);
                roomView.Refresh();
            }
        }

        private void roomView_MouseUp(object sender, MouseEventArgs e)
        {
            if (drawing != 0)
            {
                using (Graphics g = Graphics.FromImage(rooms[room])) g.DrawLine(new Pen(drawing == 1 ? colorView.BackColor : Color.Magenta), p1, p2);
                drawing = 0;
                roomView.Refresh();
                roomPalette.Refresh();
            }
        }
        #endregion
        #endregion

        #region Colors
        //Ovládání palety barev
        private void nuds_ValueChanged(object sender, EventArgs e)
        {
            colorView.BackColor = Color.FromArgb((int)nudRed.Value, (int)nudGreen.Value, (int)nudBlue.Value);
            colorView.Refresh();
        }

        private void colorAdd_Click(object sender, EventArgs e)
        {
            colors.Add(colorView.BackColor);
            if (colors.Count > 10) cOffset = colors.Count - 10;
            colorPalette.Refresh();
        }

        private void colorRemove_Click(object sender, EventArgs e)
        {
            if(colors.Contains(colorView.BackColor))
            {
                colors.Remove(colorView.BackColor);
                if (cOffset > 0 && cOffset + 10 > colors.Count) cOffset--;
                colorPalette.Refresh();
            }
        }

        private void colorPalette_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            for (int i = 0; i < (colors.Count < 10 ? colors.Count : 10); i++)
            {
                g.FillRectangle(new SolidBrush(colors[i + cOffset]), new Rectangle(20 * i, 0, 20, 20));
            }
        }

        private void colorPalette_MouseClick(object sender, MouseEventArgs e)
        {
            int m = colorPalette.PointToClient(MousePosition).X / 20 + cOffset;
            if(m < colors.Count)
            {
                Color c = colors[m];
                nudRed.Value = c.R;
                nudGreen.Value = c.G;
                nudBlue.Value = c.B;
            }
        }

        private void colorLeft_Click(object sender, EventArgs e)
        {
            Button b = sender as Button;
            if (b.Text == "←" && cOffset > 0) cOffset--;
            else if (b.Text == "→" && cOffset + 10 < colors.Count) cOffset++;
            colorPalette.Refresh();
        }
        #endregion

        #region Textures
        #region Texture management
        //Ovládání palety textur
        private void textureAdd_Click(object sender, EventArgs e)
        {
            NewTexture();
            if (textures.Count > 26) tOffset = textures.Count - 26;
            texturePalette.Refresh();
        }

        private void textureRemove_Click(object sender, EventArgs e)
        {
            textures.RemoveAt(texture);
            if (textures.Count <= 0) NewTexture();
            else if (texture >= textures.Count) texture = textures.Count - 1;
            if (tOffset > 0 && tOffset + 26 > textures.Count) tOffset--;
            textureView.Refresh();
            texturePalette.Refresh();
        }

        private void NewTexture()
        {
            Bitmap b = new Bitmap(1, 9);
            using (Graphics g = Graphics.FromImage(b)) g.Clear(Color.Gray);
            textures.Add(b);
        }

        private void texturePalette_MouseClick(object sender, MouseEventArgs e)
        {
            int m = texturePalette.PointToClient(MousePosition).X / 10 + tOffset;
            if (m < textures.Count)
            {
                texture = m;
                Color c = textures[texture].GetPixel(0, 0);
                nudRed.Value = c.R;
                nudGreen.Value = c.G;
                nudBlue.Value = c.B;
                textureView.Refresh();
            }
        }

        private void texturePalette_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;
            g.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.Half;
            for (int i = 0; i < (textures.Count < 26 ? textures.Count : 26); i++)
            {
                g.DrawImage(textures[i + tOffset], new Rectangle(10 * i, 0, 10, texturePalette.Height));
            }
        }

        private void textureLeft_Click(object sender, EventArgs e)
        {
            Button b = sender as Button;
            if (b.Text == "←" && tOffset > 0) tOffset--;
            else if (b.Text == "→" && tOffset + 26 < textures.Count) tOffset++;
            texturePalette.Refresh();
        }
        #endregion
        #region Texture painting
        //Kreslení textur
        private void textureView_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;
            g.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.Half;
            g.DrawImage(textures[texture], new Rectangle(0, 0, textureView.Width, textureView.Height));
        }

        private void textureView_MouseClick(object sender, MouseEventArgs e)
        {
            Point m = new Point(0, textureView.PointToClient(MousePosition).Y / 20);
            if (e.Button == MouseButtons.Left)
            {
                textures[texture].SetPixel(m.X, m.Y, colorView.BackColor);
                textureView.Refresh();
                texturePalette.Refresh();
            }
            else if (e.Button == MouseButtons.Right)
            {
                Color c = textures[texture].GetPixel(m.X, m.Y);
                nudRed.Value = c.R;
                nudGreen.Value = c.G;
                nudBlue.Value = c.B;
            }
        }
        #endregion
        #endregion

        #region Files
        //Uložení úrovně
        private void butSave_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Filter = "png|*.png";
            if(sfd.ShowDialog() == DialogResult.OK)
            {
                Bitmap bmp = new Bitmap(50 * rooms.Count > textures.Count + 1 ? 50 * rooms.Count : textures.Count + 1, 59);
                using (Graphics g = Graphics.FromImage(bmp)) 
                {
                    g.Clear(Color.White);
                    bmp.SetPixel(0, 0, Color.Magenta);
                    bmp.SetPixel(0, 1, Color.Red);
                    bmp.SetPixel(0, 2, Color.Lime);
                    bmp.SetPixel(0, 3, Color.Blue);
                    bmp.SetPixel(0, 4, Color.Yellow);
                    for (int i = 0; i < rooms.Count; i++)
                    {
                        g.DrawImage(rooms[i], new Point(50 * i, 9));
                    }
                    for (int i = 0; i < textures.Count; i++)
                    {
                        g.DrawImage(textures[i], new Point(1 + i, 0));
                    }
                }
                bmp.Save(sfd.FileName);
            }
        }

        //Načtení úrovně
        private void butLoad_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "png|*.png";
            if(ofd.ShowDialog() == DialogResult.OK)
            {
                rooms.Clear();
                textures.Clear();
                Bitmap bmp = new Bitmap(ofd.FileName);
                if (bmp.Width < 50 || bmp.Height != 59) MessageBox.Show("Map too small", "Error");
                else
                {
                    for (int o = 0; o < bmp.Width / 50; o++)
                    {
                        rooms.Add(bmp.Clone(new Rectangle(o * 50, 9, 50, 50), System.Drawing.Imaging.PixelFormat.Format32bppArgb));
                    }
                    int i = 1;
                    while (bmp.GetPixel(i, 0) != bmp.GetPixel(0, 5) && i < bmp.Width)
                    {
                        textures.Add(bmp.Clone(new Rectangle(1 + i, 0, 1, 9), System.Drawing.Imaging.PixelFormat.Format32bppArgb));
                        i++;
                    }
                    roomView.Refresh();
                    roomPalette.Refresh();
                    textureView.Refresh();
                    texturePalette.Refresh();
                }
            }
        }
        #endregion
    }
}
